unit gwrlr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids;

type
//
// Grid With Ruler
//////////////////////////////////////
  TGridWithRuler = class(TStringGrid)
  private
    { Private declarations }
    // HRuler
    FHRuler : Boolean;
    FHRulerWidth : Byte;
    FHRulerColor : TColor;
    FHRulerBkColor : TColor;
    FHRulerBkStyle : TBrushStyle;
    // VRuler
    FVRuler : Boolean;
    FVRulerWidth : Byte;
    FVRulerColor : TColor;
    FVRulerBkColor : TColor;
    FVRulerBkStyle : TBrushStyle;
    // common procedures
    procedure DrawRuler;
    // HRuler
    procedure SetHRuler(Rlr:Boolean);
    procedure SetHRulerWidth(RlrWidth:Byte);
    procedure SetHRulerColor(RlrColor:TColor);
    procedure SetHRulerBkColor(RlrBkColor:TColor);
    procedure SetHRulerBkStyle(RlrBkStyle:TBrushStyle);
    // VRuler
    procedure SetVRuler(Rlr:Boolean);
    procedure SetVRulerWidth(RlrWidth:Byte);
    procedure SetVRulerColor(RlrColor:TColor);
    procedure SetVRulerBkColor(RlrBkColor:TColor);
    procedure SetVRulerBkStyle(RlrBkStyle:TBrushStyle);
  protected
    { Protected declarations }
    procedure Click; override;
    procedure Paint; override;
  public
     constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    // HRuler
    property HRuler : Boolean read FHRuler write SetHRuler default True;
    property HRulerWidth : Byte read FHRulerWidth write SetHRulerWidth;
    property HRulerColor : TColor read FHRulerColor write SetHRulerColor;
    property HRulerBkColor : TColor read FHRulerBkColor write SetHRulerBkColor;
    property HRulerBkStyle : TBrushStyle read FHRulerBkStyle write SetHRulerBkStyle;
    // VRuler
    property VRuler : Boolean read FVRuler write SetVRuler default True;
    property VRulerWidth : Byte read FVRulerWidth write SetVRulerWidth;
    property VRulerColor : TColor read FVRulerColor write SetVRulerColor;
    property VRulerBkColor : TColor read FVRulerBkColor write SetVRulerBkColor;
    property VRulerBkStyle : TBrushStyle read FVRulerBkStyle write SetVRulerBkStyle;
   end;

//
// DBGrid With Ruler
//////////////////////////////////////
  TDBGridWithRuler = class(TDBGrid)
  private
    { Private declarations }
    // HRuler
    FHRuler : Boolean;
    FHRulerWidth : Byte;
    FHRulerColor : TColor;
    FHRulerBkColor : TColor;
    FHRulerBkStyle : TBrushStyle;
    // VRuler
    FVRuler : Boolean;
    FVRulerWidth : Byte;
    FVRulerColor : TColor;
    FVRulerBkColor : TColor;
    FVRulerBkStyle : TBrushStyle;
    // common procedures
    procedure DrawRuler;
    // HRuler
    procedure SetHRuler(Rlr:Boolean);
    procedure SetHRulerWidth(RlrWidth:Byte);
    procedure SetHRulerColor(RlrColor:TColor);
    procedure SetHRulerBkColor(RlrBkColor:TColor);
    procedure SetHRulerBkStyle(RlrBkStyle:TBrushStyle);
    // VRuler
    procedure SetVRuler(Rlr:Boolean);
    procedure SetVRulerWidth(RlrWidth:Byte);
    procedure SetVRulerColor(RlrColor:TColor);
    procedure SetVRulerBkColor(RlrBkColor:TColor);
    procedure SetVRulerBkStyle(RlrBkStyle:TBrushStyle);
  protected
    { Protected declarations }
    procedure Click; override;
    procedure Paint; override;
  public
     constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    // HRuler
    property HRuler : Boolean read FHRuler write SetHRuler default True;
    property HRulerWidth : Byte read FHRulerWidth write SetHRulerWidth;
    property HRulerColor : TColor read FHRulerColor write SetHRulerColor;
    property HRulerBkColor : TColor read FHRulerBkColor write SetHRulerBkColor;
    property HRulerBkStyle : TBrushStyle read FHRulerBkStyle write SetHRulerBkStyle;
    // VRuler
    property VRuler : Boolean read FVRuler write SetVRuler default True;
    property VRulerWidth : Byte read FVRulerWidth write SetVRulerWidth;
    property VRulerColor : TColor read FVRulerColor write SetVRulerColor;
    property VRulerBkColor : TColor read FVRulerBkColor write SetVRulerBkColor;
    property VRulerBkStyle : TBrushStyle read FVRulerBkStyle write SetVRulerBkStyle;
   end;


procedure Register;

implementation

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 ++            Grid With Ruler implementation              ++
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
constructor TGridWithRuler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FHRuler := True;	{ set the default values for Horizontal Ruler}
  FHRulerWidth:=3;
  FHRulerColor:=clGreen;
  FHRulerBkColor:=clYellow;
  FHRulerBkStyle:=bsClear;

  FVRuler := True;	{ set the default values for Vertical Ruler}
  FVRulerWidth:=3;
  FVRulerColor:=clGreen;
  FVRulerBkColor:=clYellow;
  FVRulerBkStyle:=bsClear;
  { I must have goEditing in Options because in readOnly Grid
    it sends Paint message before Click in mouseClick and draw first in clicked cell}
  Self.Options:=[goFixedVertLine,goFixedHorzLine,goVertLine,goHorzLine,
  goRangeSelect,goRowSizing,goColSizing,goRowMoving,goColMoving,goEditing];
end;

procedure TGridWithRuler.DrawRuler;
var OldBrush:TBrush;
    OldPen:TPen;
    TopIndent, LeftIndent : Integer;
    FixedRowsHeight, FixedColsWidth : Integer;
    LeftTopAdjust, RightDownAdjust: Integer;
    RulerWidth, RulerHeight : Integer;
    i:Integer;
begin
// save the current Brush, Pen and Font
     OldBrush:=TBrush.Create;
     OldPen:=TPen.Create;
     OldBrush.Assign(Self.Canvas.Brush);
     OldPen.Assign(Self.Canvas.Pen);
// deal witn HRuler
  if FHRuler and (Self.TopRow<=Self.Row) then
  begin
     Self.Canvas.Pen.Color:=FHRulerColor;
     Self.Canvas.Pen.Width:=FHRulerWidth;
     Self.Canvas.Brush.Color:=FHRulerBkColor;
     Self.Canvas.Brush.Style:=FHRulerBkStyle;
     // calculate the Fived Rows Height
     TopIndent:=0;
     for i:=0 to Self.FixedRows-1 do Inc(TopIndent,Self.RowHeights[i]+Self.GridLineWidth);
     // because of rows can have different heights I must calculate the height
     // of each row
     for i:=Self.TopRow to Self.Row-1 do Inc(TopIndent,Self.RowHeights[i]+Self.GridLineWidth);
     // calculate the width of ruler from widths of each col
     RulerWidth:=0;
     for i:=0 to Self.FixedCols-1 do
            Inc(RulerWidth, Self.ColWidths[i]+Self.GridLineWidth);
     for i:=Self.LeftCol to Self.ColCount-1 do
            Inc(RulerWidth, Self.ColWidths[i]+Self.GridLineWidth);

     LeftTopAdjust:=fHRulerWidth div 2;
     RightDownAdjust:=fHRulerWidth div 2+fHRulerWidth mod 2;
     Self.Canvas.Rectangle(LeftTopAdjust,TopIndent+LeftTopAdjust, RulerWidth-RightDownAdjust,TopIndent+Self.RowHeights[Self.Row]-RightDownAdjust+1);
  end;

// deal witn VRuler
  if FVRuler and (Self.LeftCol<=Self.Col) then
  begin
     Self.Canvas.Pen.Color:=FVRulerColor;
     Self.Canvas.Pen.Width:=FVRulerWidth;
     Self.Canvas.Brush.Color:=FVRulerBkColor;
     Self.Canvas.Brush.Style:=FVRulerBkStyle;
     // calculate the Fived Rows Height
     LeftIndent:=0;
     for i:=0 to Self.FixedCols-1 do Inc(LeftIndent,Self.ColWidths[i]+Self.GridLineWidth);
     // because of rows can have different heights I must calculate the height
     // of each row
     for i:=Self.LeftCol to Self.Col-1 do Inc(LeftIndent,Self.ColWidths[i]+Self.GridLineWidth);
     // calculate the width of ruler from widths of each col
     RulerHeight:=0;
     for i:=0 to Self.FixedRows-1 do
            Inc(RulerHeight, Self.RowHeights[i]+Self.GridLineWidth);
     for i:=Self.TopRow to Self.RowCount-1 do
            Inc(RulerHeight, Self.RowHeights[i]+Self.GridLineWidth);

     LeftTopAdjust:=fVRulerWidth div 2;
     RightDownAdjust:=fVRulerWidth div 2+fVRulerWidth mod 2;
     Self.Canvas.Rectangle(LeftIndent+LeftTopAdjust, LeftTopAdjust, LeftIndent+Self.ColWidths[Self.Col]-RightDownAdjust+1 ,RulerHeight-RightDownAdjust);
  end;

// restore an old Brush, Pen and Font
     Self.Canvas.Brush.Assign(OldBrush);
     Self.Canvas.Pen.Assign(OldPen);
     OldBrush.Free;
     OldPen.Free;
end;

procedure TGridWithRuler.Click;
begin
  Self.Refresh;
  inherited;
end;

procedure TGridWithRuler.Paint;
begin
  inherited;
  DrawRuler;
end;


{ HRuler private properties}
procedure TGridWithRuler.SetHRuler(Rlr:Boolean);
begin
   FHRuler:=Rlr;
   Refresh;
end;

procedure TGridWithRuler.SetHRulerWidth(RlrWidth:Byte);
begin
   FHRulerWidth:=RlrWidth;
   Refresh;
end;

procedure TGridWithRuler.SetHRulerColor(RlrColor:TColor);
begin
   FHRulerColor:=RlrColor;
   Refresh;
end;

procedure TGridWithRuler.SetHRulerBkColor(RlrBkColor:TColor);
begin
   FHRulerBkColor:=RlrBkColor;
   Refresh;
end;

procedure TGridWithRuler.SetHRulerBkStyle(RlrBkStyle:TBrushStyle);
begin
   FHRulerBkStyle:=RlrBkStyle;
   Refresh;
end;


{ VRuler private properties}
procedure TGridWithRuler.SetVRuler(Rlr:Boolean);
begin
   FVRuler:=Rlr;
   Refresh;
end;

procedure TGridWithRuler.SetVRulerWidth(RlrWidth:Byte);
begin
   FVRulerWidth:=RlrWidth;
   Refresh;
end;

procedure TGridWithRuler.SetVRulerColor(RlrColor:TColor);
begin
   FVRulerColor:=RlrColor;
   Refresh;
end;

procedure TGridWithRuler.SetVRulerBkColor(RlrBkColor:TColor);
begin
   FVRulerBkColor:=RlrBkColor;
   Refresh;
end;

procedure TGridWithRuler.SetVRulerBkStyle(RlrBkStyle:TBrushStyle);
begin
   FVRulerBkStyle:=RlrBkStyle;
   Refresh;
end;
{------------------------------------------------------------------------}

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 ++     Data Aware   DBGrid With Ruler implementation      ++
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
constructor TDBGridWithRuler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FHRuler := True;	{ set the default values for Horizontal Ruler}
  FHRulerWidth:=3;
  FHRulerColor:=clGreen;
  FHRulerBkColor:=clYellow;
  FHRulerBkStyle:=bsClear;

  FVRuler := True;	{ set the default values for Vertical Ruler}
  FVRulerWidth:=3;
  FVRulerColor:=clGreen;
  FVRulerBkColor:=clYellow;
  FVRulerBkStyle:=bsClear;
end;

procedure TDBGridWithRuler.DrawRuler;
var OldBrush:TBrush;
    OldPen:TPen;
    TopIndent, LeftIndent : Integer;
    FixedRowsHeight, FixedColsWidth : Integer;
    LeftTopAdjust, RightDownAdjust: Integer;
    RulerWidth, RulerHeight : Integer;
    i:Integer;
begin
// save the current Brush, Pen and Font
     OldBrush:=TBrush.Create;
     OldPen:=TPen.Create;
     OldBrush.Assign(Self.Canvas.Brush);
     OldPen.Assign(Self.Canvas.Pen);
// deal witn HRuler
  if FHRuler and (Self.TopRow<=Self.Row) then
  begin
     Self.Canvas.Pen.Color:=FHRulerColor;
     Self.Canvas.Pen.Width:=FHRulerWidth;
     Self.Canvas.Brush.Color:=FHRulerBkColor;
     Self.Canvas.Brush.Style:=FHRulerBkStyle;
     // calculate the Fived Rows Height
     TopIndent:=0;
     for i:=0 to Self.FixedRows-1 do Inc(TopIndent,Self.RowHeights[i]+Self.GridLineWidth);
     // because of rows can have different heights I must calculate the height
     // of each row
     for i:=Self.TopRow to Self.Row-1 do Inc(TopIndent,Self.RowHeights[i]+Self.GridLineWidth);
     // calculate the width of ruler from widths of each col
     RulerWidth:=0;
     for i:=0 to Self.FixedCols-1 do
            Inc(RulerWidth, Self.ColWidths[i]+Self.GridLineWidth);
     for i:=Self.LeftCol to Self.ColCount-1 do
            Inc(RulerWidth, Self.ColWidths[i]+Self.GridLineWidth);

     LeftTopAdjust:=fHRulerWidth div 2;
     RightDownAdjust:=fHRulerWidth div 2+fHRulerWidth mod 2;
     Self.Canvas.Rectangle(LeftTopAdjust,TopIndent+LeftTopAdjust, RulerWidth-RightDownAdjust,TopIndent+Self.RowHeights[Self.Row]-RightDownAdjust+1);
  end;

// deal witn VRuler
  if FVRuler and (Self.LeftCol<=Self.Col) then
  begin
     Self.Canvas.Pen.Color:=FVRulerColor;
     Self.Canvas.Pen.Width:=FVRulerWidth;
     Self.Canvas.Brush.Color:=FVRulerBkColor;
     Self.Canvas.Brush.Style:=FVRulerBkStyle;
     // calculate the Fived Rows Height
     LeftIndent:=0;
     for i:=0 to Self.FixedCols-1 do Inc(LeftIndent,Self.ColWidths[i]+Self.GridLineWidth);
     // because of rows can have different heights I must calculate the height
     // of each row
     for i:=Self.LeftCol to Self.Col-1 do Inc(LeftIndent,Self.ColWidths[i]+Self.GridLineWidth);
     // calculate the width of ruler from widths of each col
     RulerHeight:=0;
     for i:=0 to Self.FixedRows-1 do
            Inc(RulerHeight, Self.RowHeights[i]+Self.GridLineWidth);
     for i:=Self.TopRow to Self.RowCount-1 do
            Inc(RulerHeight, Self.RowHeights[i]+Self.GridLineWidth);

     LeftTopAdjust:=fVRulerWidth div 2;
     RightDownAdjust:=fVRulerWidth div 2+fVRulerWidth mod 2;
     Self.Canvas.Rectangle(LeftIndent+LeftTopAdjust, LeftTopAdjust, LeftIndent+Self.ColWidths[Self.Col]-RightDownAdjust+1 ,RulerHeight-RightDownAdjust);
  end;

// restore an old Brush, Pen and Font
     Self.Canvas.Brush.Assign(OldBrush);
     Self.Canvas.Pen.Assign(OldPen);
     OldBrush.Free;
     OldPen.Free;
end;

procedure TDBGridWithRuler.Click;
begin
  Self.Refresh;
  inherited;
end;

procedure TDBGridWithRuler.Paint;
begin
  inherited;
  DrawRuler;
end;


{ HRuler private properties}
procedure TDBGridWithRuler.SetHRuler(Rlr:Boolean);
begin
   FHRuler:=Rlr;
   Refresh;
end;

procedure TDBGridWithRuler.SetHRulerWidth(RlrWidth:Byte);
begin
   FHRulerWidth:=RlrWidth;
   Refresh;
end;

procedure TDBGridWithRuler.SetHRulerColor(RlrColor:TColor);
begin
   FHRulerColor:=RlrColor;
   Refresh;
end;

procedure TDBGridWithRuler.SetHRulerBkColor(RlrBkColor:TColor);
begin
   FHRulerBkColor:=RlrBkColor;
   Refresh;
end;

procedure TDBGridWithRuler.SetHRulerBkStyle(RlrBkStyle:TBrushStyle);
begin
   FHRulerBkStyle:=RlrBkStyle;
   Refresh;
end;


{ VRuler private properties}
procedure TDBGridWithRuler.SetVRuler(Rlr:Boolean);
begin
   FVRuler:=Rlr;
   Refresh;
end;

procedure TDBGridWithRuler.SetVRulerWidth(RlrWidth:Byte);
begin
   FVRulerWidth:=RlrWidth;
   Refresh;
end;

procedure TDBGridWithRuler.SetVRulerColor(RlrColor:TColor);
begin
   FVRulerColor:=RlrColor;
   Refresh;
end;

procedure TDBGridWithRuler.SetVRulerBkColor(RlrBkColor:TColor);
begin
   FVRulerBkColor:=RlrBkColor;
   Refresh;
end;

procedure TDBGridWithRuler.SetVRulerBkStyle(RlrBkStyle:TBrushStyle);
begin
   FVRulerBkStyle:=RlrBkStyle;
   Refresh;
end;

procedure Register;
begin
  RegisterComponents('JEFFGrids', [TGridWithRuler]);
  RegisterComponents('JEFFGrids', [TDBGridWithRuler]);
end;

end.
